#!/bin/bash

SCRIPTNAME="${SCRIPTNAME:-${0##*/}}"
BUILD_PATH="${SCRIPTNAME%.*}"
# BUILD_PATH="/srv/lll/vm/loader"
BINARY="binary.img"

SCRIPT_DIR="$(dirname "$(readlink -f "${0}")")"
GET_IPXE="${SCRIPT_DIR}/get_ipxe.sh"

if [ "${SCRIPTNAME}" == "${BUILD_PATH}" ]; then
    BUILD_PATH="${BUILD_PATH}_build"
fi


### [ FUNCTIONS ] ###

. functions.sh


### [ PREPARE ] ###

mkdir -p ${BUILD_PATH}/chroot

if [ $(id -u) -ne 0 ]; then
    log "Se ha de ejecutar el script como root."
    : > ${BUILD_PATH}/build.log
    su -c "${0} ${@}" -s /bin/bash | tee ${BUILD_PATH}/build.log
    EXIT_CODE=$?
    log "Log completo en: ${BUILD_PATH}/build.log"
    exit $?
fi

log "--- ENV INFO ---"
log "SCRIPT: ${SCRIPTNAME}"
log "PATH: ${BUILD_PATH}"
log "----------------"

cd ${BUILD_PATH}

log "Creando el binario"
dd if=/dev/zero of=${BINARY} count=16K bs=512


### [ BUILD ] ###

log "Indicando al sistema que lo trate como un dispositivo loop"
BINARY_DEV=$(losetup -f --show "${BINARY}")
BINARY_DEV_P1="${BINARY_DEV}p1"

log "--- DEV INFO ---"
log "BINARY_DEV: ${BINARY_DEV}"
log "BINARY_DEV_P1: ${BINARY_DEV_P1}"
log "----------------"

log "Preparándolo para ser particionado"
parted ${BINARY_DEV} mklabel msdos
parted ${BINARY_DEV} mkpart primary ext2 32k 100% -a minimal
parted ${BINARY_DEV} set 1 boot on

log "Forzando que se muestren sus particiones"
partx -a ${BINARY_DEV}

log "Formateando y montando la partición"
mkfs.ext2 ${BINARY_DEV_P1}
mount -t ext2 ${BINARY_DEV_P1} chroot

log "Añadiendo la configuración de grub e instalándolo"
mkdir -p chroot/boot/grub
cat > chroot/boot/grub/grub.cfg << EOF
set default=0
set timeout=5
menuentry "iPXE boot (shell)" {
    linux16 /boot/ipxe.lkrn
    initrd16 /boot/boot-shell.ipxe
}
menuentry "iPXE boot (autoboot)" {
    linux16 /boot/ipxe.lkrn
    initrd16 /boot/boot-autoboot.ipxe
}
menuentry "iPXE boot (chain)" {
    linux16 /boot/ipxe.lkrn
    initrd16 /boot/boot-chain.ipxe
}
menuentry "iPXE boot" {
    linux16 /boot/ipxe.lkrn
}
EOF
grub-install --boot-directory=chroot/boot/ --modules="ext2 part_msdos" ${BINARY_DEV}

# Añadimos una copia de ipxe
ls -la
if ! ${GET_IPXE} "ipxe.lkrn" "chroot/boot/"; then
    die "No se ha podido generar el kernel de ipxe."
fi

log "Añadiendo la configuración de iPXE"

cat > chroot/boot/boot-shell.ipxe << EOF
#!ipxe
:retry_dhcp
dhcp || goto retry_dhcp
echo netX/mac..........: \${netX/mac}
echo mac...............: \${mac}
echo netX/ip...........: \${netX/ip}
echo ip................: \${ip}
echo netX/netmask......: \${netX/netmask}
echo netmask...........: \${netmask}
echo gateway...........: \${gateway}
echo dns...............: \${dns}
echo domain............: \${domain}
echo dhcp-server.......: \${dhcp-server}
echo syslog............: \${syslog}
echo filename..........: \${filename}
echo next-server.......: \${next-server}
echo hostname..........: \${hostname}
echo uuid..............: \${uuid}
echo serial............: \${serial}
shell
EOF

cat > chroot/boot/boot-autoboot.ipxe << EOF
#!ipxe
autoboot ||
shell
EOF

cat > chroot/boot/boot-chain.ipxe << EOF
#!ipxe
:retry_dhcp
dhcp || goto retry_dhcp
chain http://\${dhcp-server}/pfg/auto_boot.php?mac=\${netX/mac} ||
shell
EOF

log "Iniciado la limpieza, desmontando la imagen."
umount ${BINARY_DEV_P1}

log "Desregistrando el dispositivo loop y sus particiones."
partx -d ${BINARY_DEV_P1}
losetup -d ${BINARY_DEV}

log "Corrigiendo los permisos del binario"
chmod 666 ${BINARY}


### [ TEST ] ###

if [ -e ${BINARY} ]; then
    log "Se ha creado el binario: ${BUILD_PATH}/${BINARY}"
    #~ if command_exists kvm; then
        #~ log "Iniciando con: kvm -hda ${BUILD_PATH}/${BINARY}"
        #~ kvm -hda ${BINARY} &
    #~ elif command_exists qemu; then
        #~ log "Iniciando con: qemu -hda ${BUILD_PATH}/${BINARY}"
        #~ qemu -hda ${BINARY} &
    #~ fi
else
    err "No se ha encontrado el binario."
fi
