#!/bin/bash

debug ()
{
    message debug "${@}"
}

log ()
{
    message log "${@}"
}

warn ()
{
    message warn "${@}"
}

err ()
{
    >&2 message err "${@}"
}

message ()
{
    LEVEL="${1}"
    MSG="${@:2}"
    COLOR=""
    
    echo -en "[$(date +'%Y-%m-%d %H:%M:%S')] "
    case "${LEVEL}" in
        debug)
            echo -en "[\e[1;36m${LEVEL^^}\e[0m] "
            ;;
        log)
            echo -en "[\e[1;32m${LEVEL^^}\e[0m] "
            ;;
        warn)
            echo -en "[\e[1;33m${LEVEL^^}\e[0m] "
            ;;
        err)
            echo -en "[\e[1;31m${LEVEL^^}\e[0m] "
            ;;
        *)
            echo -en "[${LEVEL^^}] "
    esac
    echo "${MSG}"
}

die ()
{
    err "${@}"
    exit 1
}

hr ()
{
    printf "%${COLUMNS}s\n" | tr " " "-"
}

function_exists()
{
    declare -f "${1}" > /dev/null
}

command_exists ()
{
    command -v "${1}" > /dev/null 2>&1
}

pkg_exists ()
{
    if command_exists dpkg; then
        dpkg -s "${1}" > /dev/null 2>&1
    elif command_exists pacman; then
        pacman -Qs "^${1}$" > /dev/null 2>&1
    fi
}

# src: https://gist.github.com/davejamesmiller/1965569
ask ()
{
    while :; do
        
        if [ "${2:-}" = "Y" ]; then
            prompt="Y/n"
            default=Y
        elif [ "${2:-}" = "N" ]; then
            prompt="y/N"
            default=N
        else
            prompt="y/n"
            default=
        fi
        
        # Ask the question
        read -n 1 -p "$1 [$prompt]: " REPLY
        
        # Default?
        if [ -z "$REPLY" ]; then
            REPLY=$default
        else
            echo
        fi
        
        # Check if the reply is valid
        case "$REPLY" in
            Y*|y*) return 0 ;;
            N*|n*) return 1 ;;
        esac
        
    done
}
