/**
 * BuildController
 *
 * @description :: Server-side logic for managing builds
 * @help        :: See http://links.sailsjs.org/docs/controllers
 */

var spawn = require('child_process').spawn;

module.exports = {
    
    index: function (req, res, next) {
        var activeConnector = req.param('connector');
        var connectorParams = activeConnector ? Connector.get(activeConnector) : {};
        
        return res.view({
            connectors: Connector.list(),
            activeConnector: activeConnector,
            connectorParams: connectorParams.env || {}
        });
    },
    
    start: function (req, res, next) {
        var connector = req.param('connector'),
            launcher = '/srv/nbm/utils/launch_script.sh';
            connectorParams = _.pick(
                req.params.all(), _.keys(Connector.get(connector).env || {}));
        
        // Get build script
        return GenScript.build(connector, connectorParams, function scriptRendered(err, buildName, result) {
            if ( err ) {
                sails.log.error(err);
                return next(err);
            }
            
            // Create build record
            Build.create({
                name: buildName,
                status: 'building',
                log: '',
                image: { name: buildName, connector: connector, build: buildName, hidden: true }
            }, function (err, b) {
                if ( err ) {
                    sails.log.error(err);
                    return next(err);
                }
                
                BinaryImage.update(b.image.id, { log: b.id }, function (err, next) {});
                
                // Start root shell
                var shell = spawn(launcher, [], {
                    detached: true,
                });
                
                // Create build room
                var socketRoom = 'build' + buildName;
                
                // Show console on client
                res.redirect('/build/show/' + b.id);
                
                // Configure bash fd's
                shell.stdout.on('data', function (data) {
                    sails.sockets.broadcast(socketRoom, 'consoleMessage', {
                        fd: 'stdout',
                        msg: String(data).trim()
                    });
                    b.log += String(data);
                    // sails.log.info(String(data).trim());
                });
                
                shell.stderr.on('data', function (data) {
                    sails.sockets.broadcast(socketRoom, 'consoleMessage', {
                        fd: 'stderr',
                        msg: String(data).trim()
                    });
                    b.log += String(data);
                    // sails.log.error(String(data).trim());
                });
                
                shell.on('close', function (code) {
                    var closeMsg = 'Process finished with code: ' + code +'.';
                    
                    // Remove process list
                    Build.stopProcess(b.id, shell);
                    
                    sails.sockets.broadcast(socketRoom, 'consoleMessage', {
                        fd: code !== 0 ? 'stderr' : 'stdout',
                        msg: closeMsg
                    });
                    sails.log.debug(closeMsg);
                    b.log += closeMsg;
                    
                    if ( code !== 0 ) {
                        b.status = 'failed';
                    } else {
                        b.status = 'succeeded';
                        BinaryImage.update(b.image.id, { hidden: false }, function (err, next) {});
                    }
                    
                    b.save(function (err) {
                        if ( err ) {
                            sails.log.error(err);
                            sails.log.error('Unable to store build log from build `' + buildName + '`. Dumping...' + "\n" + b.log);
                        } else {
                            sails.log.info('Build `' + buildName + '` correctly logged.');
                        }
                    });
                });
                
                // Add to process list
                Build.addProcess(b.id, shell);
                
                // Pipe to shell
                shell.stdin.write(result);
                shell.stdin.end();
            });
        });
    },
    
    stop: function (req, res, next) {
        var id = req.param('id');
        
        if ( id ) {
            // Remove process list
            Build.stopProcess(id);
        }
    },
    
    show: function (req, res, next) {
        var id = req.param('id');
        
        if ( id ) {
            Build.findOne(id, function (err, build) {
                if ( err ) {
                    sails.log.error(err);
                    return next(err);
                }
                
                return res.view({
                    build: build
                });
            });
        } else {
            return res.notFound();
        }
    },
    
    subscribe: function (req, res, next) {
        var build = req.param('build');
        
        // Subscribe user to `builder` room
        if ( req.isSocket && build ) {
            sails.sockets.join(req.socket, 'build' + build);
            return res.ok();
        }
        return res.badRequest();
    }
};
