/**
 * CoreController
 *
 * @description :: Server-side logic for managing cores
 * @help        :: See http://links.sailsjs.org/docs/controllers
 */

var fs = require('fs');

module.exports = {
    
    boot: function (req, res, next) {
        GenScript.boot(function scriptRendered(err, result) {
            // Log any error and exit
            if ( err ) {
                sails.log.error(err);
                return next(err);
            }
            
            res.type('text/plain');
            return res.send(result);
        });
    },
    
    getscript: function (req, res, next) {
        var mac = req.param('mac'),
            mac_re = /^([0-9A-Fa-f]{2}:){5}([0-9A-Fa-f]{2})$/;
        
        // If wrong MAC, exit
        if ( ! mac_re.test(mac) ) {
            var wrongMacFormatError = {
                name: 'wrongMacFormatError',
                message: 'The supplied MAC address isn\'t correctly formatted: `' + mac + '`.'
            };
            return next(wrongMacFormatError);
        }
        
        Client.findOrCreate({mac: mac}, {
            name: mac.toLowerCase().replace(/[^a-z0-9-]/gi, '-'), // RFC 1035 compliant
            mac: mac,
            ip: req.ip
        }).exec(function clientFound(err, client) {
            // Log any error and exit
            if ( err ) {
                sails.log.error(err);
                return next(err);
            }
            
            GenScript.ipxe(client.id, function scriptRendered(err, result) {
                // Log any error and exit
                if ( err ) {
                    sails.log.error(err);
                    return next(err);
                }
                
                sails.sockets.broadcast('dashboard', 'clientConnected', {
                    id: client.id,
                    name: client.name,
                    mac: client.mac,
                    ip: req.ip,
                    images: imgs.length,
                    dhcp: client.dhcp ? ( req.ip === client.ip ? 'fixed' : 'error' ) : 'auto',
                    timestamp: new Date()
                });
                
                res.type('text/plain');
                return res.send(result);
            });
        });
    },
    
    getfile: function (req, res, next) {
        var id = req.param('id');
        // Get file and avoid path injection
        var file = (req.param('file') || '').split("/").pop();
        
        // sails.log.debug('*** getfile ***');
        
        // Avoid file name path injection
        if ( id && file && file != '.' && file != '..' ) {
            BinaryImage.findOne(id).exec(function imageFound(err, img) {
                // Log any error and exit
                if ( err ) {
                    sails.log.error(err);
                    return next(err);
                }
                
                var connector = Connector.get(img.connector);
                
                if ( ! connector ) {
                    var incorrectConnectorError = {
                        name: 'incorrectConnectorError',
                        message: 'Unable to get connector: `' + img.connector + '`.'
                    };
                    return next(incorrectConnectorError);
                }
                
                var path = connector.path + '/' + img.build + '/' + file;
                
                // Read file and send it
                fs.readFile(path, function (err, data) {
                    // Log any error and exit
                    if ( err ) {
                        // sails.log.error(err); // Error logged by default
                        return next(err);
                    }
                    
                    // sails.log.debug('Data length: ' + data.length);
                    // Set Content-Disposition and Content-Type headers
                    // @see: http://expressjs.com/api#res.attachment
                    res.attachment(path);
                    return res.send(data);
                });
            });
        } else {
            var wrongFileNameError = {
                name: 'wrongFileNameError',
                message: 'The requested file name is forbidden: `' + file + '`.'
            };
            return next(wrongFileNameError);
        }
    },
    
    gendhcpd: function (req, res, next) {
        var file = '/srv/nbm/dhcpd.conf';
        
        return GenScript.dhcpd(function scriptRendered(err, result) {
            if ( err ) {
                return next(err);
            }
            
            fs.writeFile(file, result, function (err) {
                if ( err ) {
                    return next(err);
                }
                
                sails.log.info('`' + file + '` successfully updated. Please, restart DHCPD Server to apply changes.');
                res.type('text/plain');
                return res.send(result);
            });
            
        });
    },
    
    genexports: function (req, res, next) {
        var file = '/srv/nbm/exports';
        
        return GenScript.exports(function scriptRendered(err, result) {
            if ( err ) {
                return next(err);
            }
            
            fs.writeFile(file, result, function (err) {
                if ( err ) {
                    return next(err);
                }
                
                sails.log.info('`' + file + '` successfully generated. Please, restart NFS Server or reload exports with `exportfs -rav` to apply changes.');
                res.type('text/plain');
                return res.send(result);
            });
            
        });
    },
    
    genbuild: function (req, res, next) {
        var connector = req.param('connector');
        
        return GenScript.build(connector, {}, function scriptRendered(err, build, result) {
            if ( err ) {
                return next(err);
            }
            
            sails.log.info('Connector `' + connector + '` successfully generated for build `' + build + '`.');
            res.type('text/plain');
            return res.send(result);
        });
    },
    
};
