/**
 * IndexController
 *
 * @description :: Server-side logic for managing indices
 * @help        :: See http://links.sailsjs.org/docs/controllers
 */

module.exports = {
    
    /** 
     * First and default web view
     */
    index: function (req, res, next) {
        if ( req.session.userId ) {
            // Index.index doesn's require sessionAuth, so session.User is not
            // reloaded and navbar is broken
            return res.redirect('/dashboard');
        } else {
            return res.view();
        }
    },
    
    /**
     * Sets the maxium records to display at once.
     */
    limit: function (req, res, next) {
        var model = req.param('model');
        var limit = parseInt(req.param('limit'), 10);
        
        req.session.management = req.session.management || {};
        req.session.management[model] = req.session.management[model] || {};
        
        req.session.management[model].limit = limit;
        return res.ok();
    },
    
    /**
     * Sets the records display order.
     */
    sort: function (req, res, next) {
        var model = req.param('model');
        var dir = req.param('dir');
        var col = req.param('col');
        
        if ( ! _.contains(['asc', 'desc'], dir) ) {
            dir = 'asc';
        }
        
        if ( ! col ) {
            var wrongColumnName = {
                name: 'wrongColumnName',
                message: 'Wrong column name: `' + col + '`.'
            };
            return next(wrongColumnName);
        }
        
        req.session.management = req.session.management || {};
        req.session.management[model] = req.session.management[model] || {};
        
        req.session.management[model].sort = {};
        req.session.management[model].sort[col] = dir;
        
        // sails.log.debug('management', req.session.management);
        return res.ok();
    },
    
};
