/**
 * UserController
 *
 * @description :: Server-side logic for managing users
 * @help        :: See http://links.sailsjs.org/docs/controllers
 */

var ControllerBase = require('./ControllerBase');
var self = _.cloneDeep(ControllerBase);

module.exports = _.merge(self, {
    
    _model: function () { return User; },
    _controller: function () { return 'user'; },
    _attributes: function () { return ['name', 'fullName', 'email', 'roles', 'password']; },
    
    /** 
     * Show new record form
     */
    'new': function (req, res, next) {
        self._manageForm(req, res, next, 'create');
    },
    
    /** 
     * Show record form (readonly)
     */
    show: function (req, res, next) {
        self._manageForm(req, res, next, 'show');
    },
    
    /** 
     * Show record form (editable)
     */
    edit: function (req, res, next) {
        self._manageForm(req, res, next, 'update');
    },
    
    /**
     * Show 'manage/form.ejs' to create/show/edit Model
     *
     * @param {string} action - Form's action.
     */
    _manageForm: function (req, res, next, action) {
        var id = _.indexOf(['show', 'update'], action) > -1 ? req.param('id') : null;
        var fields = _.cloneDeep(self._model().formFields);
        
        // Obtenemos la lista total de roles
        Role.find().sort({ name: 'asc' })
        .exec(function rolesFound(err, roles) {
            // Si ha ocurrido algun error, lo devolvemos
            if ( err ) {
                return next(err);
            }
            
            // Inyectamos los roles
            var rolesField = _.find(fields, function (f) { return f.name === 'roles'; }) || {};
            rolesField.values = roles;
            
            if ( id ) {
                // Buscamos el usuario dada su ID
                self._model().findOne(id).populate('roles')
                .exec(function recordFound(err, record) {
                    // Si ha ocurrido algun error, lo devolvemos
                    if ( err ) {
                        return next(err);
                    }
                    
                    // Si el rol no existe, salimos
                    if ( ! record ) {
                        var recordNotFoundError = {
                            name: 'recordNotFoundError',
                            message: 'No record found for `' + self._controller() + '` with id `' + id + '`.'
                        };
                        return next(recordNotFoundError);
                    }
                    
                    // Merge unsaved fields, if any
                    self._mergeObjState(self, record, req);
                    
                    // Pasamos las variables al template
                    return res.view('model/form', {
                        model: {
                            name: self._controller(),
                            action: action
                        },
                        record: record.toJSON(),
                        fields: fields
                    });
                });
            } else {
                // Merge unsaved fields, if any
                var record = {};
                self._mergeObjState(self, record, req);
                
                // Pasamos las variables al template
                return res.view('model/form', {
                    model: {
                        name: self._controller(),
                        action: action
                    },
                    record: record,
                    fields: fields
                });
            }
        });
    },
    
    /** 
     * Creates a new record.
     */
    create: function (req, res, next) {
        self._manageFormAction(req, res, next);
    },
    
    /** 
     * Edit an existing record.
     */
    update: function (req, res, next) {
        self._manageFormAction(req, res, next);
    },
    
    
    /**
     * [ Inherited methods ]
     */
     
    index: function (req, res, next) {
        ControllerBase.index.call(self, req, res, next);
    },
    
    destroy: function (req, res, next) {
        ControllerBase.destroy.call(self, req, res, next);
    },
    
    destroyMultiple: function (req, res, next) {
        ControllerBase.destroyMultiple.call(self, req, res, next);
    },
    
    _manageFormAction: function (req, res, next) {
        ControllerBase._manageFormAction.call(self, req, res, next);
    },
});
