/**
* Image.js
*
* @description :: TODO: You might write a short summary of how this model works and what it represents here.
* @docs        :: http://sailsjs.org/#!documentation/models
*/

var sprintf = require('sprintf').sprintf;

module.exports = {
    
    attributes: {
        
        /**
        * Nombre de la imagen.
        */
        name: {
            type: 'string',
            unique: true,
            required: true,
            notNull: true
        },
        
        /**
         * El tipo de la imagen (función y FS)
         */
        connector: {
            type: 'string',
            required: true
        },
        
        /**
         * Ruta base (absoluta).
         */
        build: {
            type: 'string',
            required: true
        },
        
        /**
         * Grupos con acceso a la imagen.
         */
        groups: {
            collection: 'clientGroup',
            via: 'images'
        },
        
        /**
         * Toggle image visibility when building the script
         */
        hidden: {
            type: 'boolean',
            defaultsTo: false
        },
        
        /**
         * Image priority when building script
         */
        priority: {
            type: 'integer',
            defaultsTo: 0
        },
        
        // Linked build, if any
        log: {
            model: 'build',
        },
        
        /**
         * Return BinaryImage URL.
         *
         * @param {string} file - File to get
         * @return {string} Full URL to file.
         */
        url: function () {
            // ${dhcp-server} is assigned by iPXE
            var server_ip = Config.getParam('server_ip', '${dhcp-server}');
            var server_port = Config.getParam('server_port', 1337);
            return sprintf('http://%s:%s/getfile/%s', server_ip, server_port, this.id);
        }
    },
    
    beforeValidate: function (values, next) {
        // sails.log.debug('values', values);
        
        if ( values.groups ) {
            values.groups = _.compact(values.groups);
        }
        
        if ( values.connector ) {
            if ( ! Connector.exists(values.connector) ) {
                var incorrectConnectorError = {
                    name: 'incorrectConnectorError',
                    message: 'Unable to set connector: `' + value.connector + '`.'
                };
                return next(incorrectConnectorError);
            } else if ( Array.isArray(values.connector) ) {
                values.connector = values.connector.slice(0, 1).pop();
            }
        }
        // sails.log.debug('post', values);
        return next();
    },
    
    /**
     * Form builder
     */
    formFields: [{
        name: 'id',
        label: 'Id',
        type: 'text',
    }, {
        name: 'name',
        label: 'Name*',
        type: 'text',
        attrs: {
            'data-editable': true,
            'data-bv-stringlength': 'true',
            'data-bv-stringlength-min': 3,
            'data-bv-notempty': 'true',
        }
    }, {
        name: 'connector',
        label: 'Connector*',
        type: 'select',
        multiple: false,
        attrs: {
            'data-editable': true,
            'data-bv-notempty': 'true',
            'placeholder': 'Connector to use.',
        },
        perms: 'manageConnectors',
        values: {}
    }, {
        name: 'build',
        label: 'Build*',
        type: 'text',
        attrs: {
            'data-editable': true,
            'data-bv-notempty': 'true',
            'placeholder': 'Build\'s folder name.',
        }
    }, {
        name: 'groups',
        label: 'Groups',
        relation: 'clientgroup',
        type: 'select',
        multiple: true,
        attrs: {
            'data-editable': true,
            'title': 'No groups assigned',
        },
        perms: 'manageGroups',
        // Must be injected: { id: '...', name: '...' }
        values: {}
    }, {
        name: 'priority',
        label: 'Priority*',
        type: 'number',
        attrs: {
            'data-editable': true,
            'data-bv-notempty': 'true',
            'data-bv-integer': 'true',
            'data-bv-between': 'true',
            'data-bv-between-inclusive': 'true',
            'data-bv-between-min': '0',
            'data-bv-between-max': '1000',
        },
    }, {
        name: 'hidden',
        label: 'Hidden',
        type: 'checkbox',
        attrs: {
            'data-editable': true,
        },
    }, {
        name: 'log',
        label: 'Log',
        relation: 'build',
        type: 'select',
        attrs: {
            'title': 'No logs found',
        },
        // Must be injected: { id: '...', name: '...' }
        values: {}

    }],
    
    /**
     * List Builder. Max col-md-10
     */
    listFields: [{
        name: 'id',
        label: '#',
        'class': 'col-md-1'
    }, {
        name: 'name',
        label: 'Name',
        'class': 'col-md-9'
    }]
    
};

