/**
 * sessionAuth
 *
 * @module      :: Policy
 * @description :: Simple policy to allow any authenticated user
 *                 Assumes that your login action in one of your controllers sets `req.session.authenticated = true;`
 * @docs        :: http://sailsjs.org/#!documentation/policies
 *
 */
module.exports = function(req, res, next) {
    sails.log.debug('Policy: sessionAuth');
    
    // Si existe userId hay un login activo. Recogemos el usuario (no podemos
    // almacenarlo porque perdemos acceso a las funciones del Model)
    if ( req.session.userId ) {
        User.getWithPerms(req.session.userId).then(function userFound(user) {
            if ( ! user.hasPerm('webAccess') ) {
                var webAccessDisabledError = {
                    context: 'danger',
                    title: 'Disabled web access',
                    message: 'User \'' + user.name + '\' is not allowed to access web.'
                };
                req.session.flash = {
                    alerts: [webAccessDisabledError]
                };
                delete req.session.userId;
                return res.redirect('/');
                // return next(webAccessDisabledError);
            }
            // Remove password, store `user` and continue
            delete user.encryptedPassword;
            req.session.User = user;
            return next();
        }).fail(function userNotFound(err) {
            var userNotFoundError = {
                context: 'danger',
                title: 'User not found',
                message: 'Current session\'s user `' + req.session.userId + '` not found.'
            };
            req.session.flash = {
                alerts: [userNotFoundError]
            };
            return res.redirect('/');
            // return next(userNotFoundError);
        });
    } else {
        // Acceso restringido
        var requireLoginError = {
            context: 'danger',
            title: 'Required login',
            message: 'You must sign-in to view this content.'
        };
        req.session.flash = {
            alerts: [requireLoginError],
            url: req.originalUrl
        };
        return res.redirect('/');
        // return next(requireLoginError);
    }
};
