/**
 * Niobium Globals, accesible through 'sails' global:
 * view/service/model/controller - sails.config.niobium
 */

module.exports.niobium = {
    
    /**
     * Default Niobium config
     */
    defaults: {
        'server_ip': '10.42.0.1',
        'server_port': '1337',
        'dhcp_subnet': '10.42.0.0',
        'dhcp_mask': '255.255.255.0',
        'dhcp_range': '10.42.0.10 10.42.0.30',
        'dhcp_broadcast': '10.42.0.255',
        'dhcp_dns': '10.42.0.1, 8.8.8.8, 8.8.4.4',
        'ipxe_menu_timeout': '30000',
        'ipxe_tftp_kernel': 'undionly.kpxe',
    },
    
    /**
     * Web pages schema
     * Active & hidden will be injected, if necessary.
     */
    schema: [{
        href: '/dashboard',
        route: '/dashboard',
        label: 'Dashboard',
        'class': 'glyphicon glyphicon-eye-open'
    }, {
        href: '#',
        label: 'Management',
        'class': 'glyphicon glyphicon-tasks',
        perms: 'manageUsers manageRoles managePermissions manageClients manageImages manageGroups manageConnectors',
        anyPerm: true,
        sub: [{
            href: '/user',
            route: '/user/list/:page?',
            label: 'Users',
            perms: 'manageUsers',
            sub: [
                { route: '/user/show/:id?', label: 'Show' },
                { route: '/user/edit/:id?', label: 'Edit' },
                { route: '/user/new/:id?' , label: 'Create' }
            ]
        }, {
            href: '/role',
            route: '/role/list/:page?',
            label: 'Roles',
            perms: 'manageRoles',
            sub: [
                { route: '/role/show/:id?', label: 'Show' },
                { route: '/role/edit/:id?', label: 'Edit' },
                { route: '/role/new/:id?' , label: 'Create' }
            ]
        }, {
            href: '/permission',
            route: '/permission/list/:page?',
            label: 'Permissions',
            perms: 'managePermissions',
            sub: [
                { route: '/permission/show/:id?', label: 'Show' },
                { route: '/permission/edit/:id?', label: 'Edit' },
                { route: '/permission/new/:id?' , label: 'Create' }
            ]
        }, {
            href: '/binaryimage',
            route: '/binaryimage/list/:page?',
            label: 'Images',
            perms: 'manageImages',
            sub: [
                { route: '/binaryimage/show/:id?', label: 'Show' },
                { route: '/binaryimage/edit/:id?', label: 'Edit' },
                { route: '/binaryimage/new/:id?' , label: 'Create' }
            ]
        }, {
            href: '/clientgroup',
            route: '/clientgroup/list/:page?',
            label: 'Groups',
            perms: 'manageGroups',
            sub: [
                { route: '/clientgroup/show/:id?', label: 'Show' },
                { route: '/clientgroup/edit/:id?', label: 'Edit' },
                { route: '/clientgroup/new/:id?' , label: 'Create' }
            ]
        }, {
            href: '/client',
            route: '/client/list/:page?',
            label: 'Clients',
            perms: 'manageClients',
            sub: [
                { route: '/client/show/:id?', label: 'Show' },
                { route: '/client/edit/:id?', label: 'Edit' },
                { route: '/client/new/:id?' , label: 'Create' }
            ]
        }, {
            href: '/build',
            route: '/build',
            label: 'Builds',
            perms: 'manageImages',
            sub: [
                { route: '/build/show/:id', label: 'Show' }
            ],
        }]
    }, {
        href: '#',
        label: 'Config',
        'class': 'glyphicon glyphicon-cog', // wrench
        perms: 'manageConfig',
        anyPerm: true,
        sub: [{
            href: '/config/niobium',
            route: '/config/niobium',
            label: 'Niobium'
        }, {
            href: '/config/dhcpd',
            route: '/config/dhcpd',
            label: 'DHCP Server'
        }, {
            href: '/config/nfs',
            route: '/config/nfs',
            label: 'NFS Server'
        }]
    // }, {
    //     href: '#',
    //     label: 'Help',
    //     'class': 'glyphicon glyphicon-info-sign'
    }],
    
};
